unit Figure;

interface

uses
  Math, Graphics, Controls, SysUtils, Forms, ExtCtrls, StdCtrls;

type
  TFigure=class
    protected
      x,y,w,h:integer;  //  
      img:TImage;       // Image
      color:TColor;     // 
    public
      constructor Figure(imgsrc:TImage;c:TColor;ax,ay:integer);
      procedure limXY(w:integer);
      procedure SendMsg;
      function GetX: integer;
      function GetY: integer;
      procedure SetXY(ax, ay: integer);
      procedure Show; virtual; abstract;
      procedure MoveTo(ax, ay: integer); virtual; abstract;
  end;

  TCircle=class(TFigure)
    protected
      r: integer;       //
    public
      constructor Circle(imgsrc:TImage;c:TColor;ax,ay,ar:integer);
      procedure limR;
      procedure SendMsg;
      function GetR:integer;
      procedure SetR(ar:integer);
      procedure Show;virtual;
      procedure MoveTo(ax,ay:integer);virtual;
      procedure Resize(ar:integer);
      procedure Delete;
  end;

  TEllipse=class(TCircle)
    private
      r1:integer;   // 
    public
      constructor Ellipse(imgsrc:TImage;c:TColor;ax,ay,ar,ar1: integer);
      procedure SendMsg;
      function GetR1:integer;
      procedure SetR1(ar1:integer);
      procedure Rotate90;
      procedure Show;virtual;
      procedure MoveTo(ax,ay:integer);virtual;
      procedure Resize(ar,ar1:integer);
      procedure Delete;
  end;

  TQuad=class(TFigure)
    protected
      x1,y1:integer;  //   
    public
      constructor Quad(imgsrc:TImage;c:TColor;ax,ay,ax1,ay1:integer);
      procedure limWH;
      procedure limXY(w,h:integer);
      procedure SendMsg;
      function GetX1: integer;
      function GetY1: integer;
      procedure setX1Y1(ax1,ay1:integer);
  end;

  TRectangle=class(TQuad)
    public
      constructor Rectangle(imgsrc:TImage;c:TColor;ax,ay,dx,dy:integer);
      procedure SendMsg;
      procedure Show;virtual;
      procedure MoveTo(ax, ay: integer);virtual;
      procedure Resize(dx,dy:integer);
      procedure Delete;
  end;

  TDiamond=class(TQuad)
    public
      constructor Diamond(imgsrc:TImage;c:TColor;ax,ay,dx,dy:integer);
      procedure SendMsg;
      procedure Show;virtual;
      procedure MoveTo(ax, ay: integer);virtual;
      procedure Resize(dx,dy:integer);
      procedure Delete;
  end;

  TTrapezium=class(TQuad)
    protected
      dx1,dx2:integer;
    public
      constructor Trapezium(imgsrc:TImage;c:TColor;ax,ay,dx,dy,adx1,adx2: integer);
      procedure SendMsg;
      procedure Show;virtual;
      procedure MoveTo(ax, ay: integer);virtual;
      procedure setDx1Dx2(adx1,adx2:integer);
      procedure Resize(dx,dy,dx1,dx2:integer);
      procedure Delete;
  end;

implementation

// Figure
//    
constructor TFigure.Figure(imgsrc:TImage;c:TColor;ax,ay:integer);
begin
  x:=ax;
  y:=ay;
  img:=imgsrc;
  color:=c;
  SendMsg;
end;

//.      
procedure TFigure.limXY(w:integer);
begin

  if (x+w>=img.Width) then
    x:=img.Width-w;

  if (x-w<0) then
    x:=w;

  if (y+w>=img.Height) then
    y:=img.Height-w;

  if (y-w<0) then
    y:=w;
end;

//.     
procedure TFigure.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+'}.';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.   x
function TFigure.GetX: integer;
begin
  result:=x;
end;

//.   y
function TFigure.GetY: integer;
begin
  result:=y;
end;

//.   
procedure TFigure.SetXY(ax, ay: integer);
begin
  x:=ax;
  y:=ay;
end;

// Circle
//   
constructor TCircle.Circle(imgsrc:TImage;c:TColor;ax,ay,ar:integer);
begin
  inherited Figure(imgsrc,c,ax,ay);
  r:=ar;
  limR;
  SendMsg;
end;

//.       
procedure TCircle.limR;
begin
  if (x+r>img.Width) then
    r:=img.Width-x;

  if (y+r>img.Height) then
    r:=img.Height-y;

  if (x-r<0) then
    r:=x;

  if (y-r<0) then
    r:=y;
end;

//.     
procedure TCircle.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+'}   '+IntToStr(r)+'.';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.   
function TCircle.GetR:integer;
begin
  result:=r;
end;

//.   
procedure TCircle.SetR(ar:integer);
begin
  r:=ar;
  limR;
end;

//.  
procedure TCircle.Show;
begin
  img.Canvas.Pen.Color:=color;
  img.Canvas.Ellipse(x-r, y-r, x+r, y+r);
end;

//.  
procedure TCircle.MoveTo(ax, ay: integer);
begin
  Show;
  SetXY(x+ax,y+ay);
  limXY(r);
  Show;
end;

//.  
procedure TCircle.Resize(ar: integer);
begin
  Show;
  SetR(ar);
  Show;
end;

//.  
procedure TCircle.Delete;
begin
  Show;
end;

// Ellipse
//   
constructor TEllipse.Ellipse(imgsrc:TImage;c:TColor;ax,ay,ar,ar1:integer);
begin
  inherited Circle(imgsrc,c,ax,ay,ar);
  r1:=ar1;
  limR;
  SendMsg;
end;

//.     
procedure TEllipse.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+'},  '+IntToStr(r)+'  '+IntToStr(r1)+'.';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.   
function TEllipse.GetR1: integer;
begin
  result:=r1;
end;

//.   
procedure TEllipse.SetR1(ar1: integer);
begin
  r1:=ar1;
end;

//.  
procedure TEllipse.Show;
begin
  img.Canvas.Pen.Color:=color;
  img.Canvas.Ellipse(x-r1, y-r, x+r1, y+r);
end;

//.  
procedure TEllipse.MoveTo(ax, ay: integer);
begin
  Show;
  SetXY(x+ax,y+ay);
  limXY(r);
  Show;
end;

//.  
procedure TEllipse.Resize(ar,ar1:integer);
begin
  Show;
  Rotate90;
  SetR1(ar1);
  SetR(ar);
  Show;
end;

//.    90 
procedure TEllipse.Rotate90;
var
  tmp:integer;
begin
  tmp:=r1;
  r1:=r;
  r:=tmp;
end;

//.  
procedure TEllipse.Delete;
begin
  Show;
end;

// Quad
//   
constructor TQuad.Quad(imgsrc:TImage;c:TColor;ax,ay,ax1,ay1:integer);
begin
  inherited Figure(imgsrc,c,ax,ay);
  x1:=ax+ax1;
  y1:=ay+ay1;
  SendMsg;
end;

//.       
procedure TQuad.limXY(w,h:integer);
begin

  if (x+w>=img.Width) then
  begin
    x:=img.Width-w;
    x1:=img.Width-1;
  end;

  if (x<1) then
  begin
    x:=1;
    x1:=w;
  end;

  if (y+h>=img.Height) then
  begin
    y:=img.Height-h;
    y1:=img.Height-1;
  end;

  if (y<5) then
  begin
    y:=1;
    y1:=h;
  end;
end;

//.         
procedure TQuad.limWH;
begin
  if (x1-x>=img.Width) then
    x1:=img.Width-1;

  if (y1-y>=img.Height) then
    y1:=img.Height-1;
end;

//.     
procedure TQuad.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+','+IntToStr(x1)+','+IntToStr(y1)+').';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.   x1
function TQuad.GetX1: integer;
begin
  result:=x1;
end;

//.   y1
function TQuad.GetY1: integer;
begin
    result:=y1;
end;

//.   x1  y1
procedure TQuad.setX1Y1(ax1,ay1:integer);
begin
  x1:=ax1;
  y1:=ay1;
end;

// Rectangle
//   
constructor TRectangle.Rectangle(imgsrc:TImage;c:TColor;ax,ay,dx,dy: integer);
begin
    inherited Quad(imgsrc,c,ax,ay,dx,dy);
    SendMsg;
end;

//.     
procedure TRectangle.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+','+IntToStr(x1)+','+IntToStr(y1)+').';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.  
procedure TRectangle.Show;
begin
  img.Canvas.Pen.Color:=color;
  img.Canvas.MoveTo(x,y);
  img.Canvas.LineTo(x1,y);
  img.Canvas.LineTo(x1,y1);
  img.Canvas.LineTo(x,y1);
  img.Canvas.LineTo(x,y);
end;

//.  
procedure TRectangle.MoveTo(ax, ay: integer);
begin
  Show;
  setXY(x+ax,y+ay);
  setX1Y1(x1+ax,y1+ay);
  limXY(x1-x,y1-y);
  Show;
end;

//.  
procedure TRectangle.Resize(dx,dy:integer);
begin
  Show;
  setX1Y1(x1+dx,y1+dy);
  limWH;
  Show;
end;

//.  
procedure TRectangle.Delete;
begin
  Show;
end;

// Diamond
//   
constructor TDiamond.Diamond(imgsrc:TImage;c:TColor;ax,ay,dx,dy: integer);
begin
    inherited Quad(imgsrc,c,ax,ay,dx,dy);
    SendMsg;
end;

//.     
procedure TDiamond.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+','+IntToStr(x1)+','+IntToStr(y1)+').';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.  
procedure TDiamond.Show;
begin
  img.Canvas.Pen.Color:=color;
  img.Canvas.MoveTo(x,y);
  img.Canvas.LineTo(x1,y1);
  img.Canvas.LineTo(x,y1+abs(y1-y));
  img.Canvas.LineTo(x-abs(x1-x),y1);
  img.Canvas.LineTo(x,y);
end;

//.  
procedure TDiamond.MoveTo(ax, ay: integer);
begin
  Show;
  setXY(x+ax,y+ay);
  setX1Y1(x1+ax,y1+ay);
  limXY(x1-x,y1-y);
  Show;
end;

//.  
procedure TDiamond.Resize(dx,dy:integer);
begin
  Show;
  setX1Y1(x1+dx,y1+dy);
  limWH;
  Show;
end;

//.  
procedure TDiamond.Delete;
begin
  Show;
end;

// Trapezium
//   
constructor TTrapezium.Trapezium(imgsrc:TImage;c:TColor;ax,ay,dx,dy,adx1,adx2: integer);
begin
    inherited Quad(imgsrc,c,ax,ay,dx,dy);
    dx1:=adx1;
    dx2:=adx2;
    SendMsg;
end;

//.     
procedure TTrapezium.SendMsg;
var
  msg:string;
begin
  msg:= '    ('+IntToStr(x)+','+IntToStr(y)+','+IntToStr(x1)+','+IntToStr(y1)+').';
  ((img.GetParentComponent As TForm).FindChildControl('MsgWin') As TMemo).Lines.Add(msg);
end;

//.  
procedure TTrapezium.Show;
begin
  img.Canvas.Pen.Color:=color;
  img.Canvas.MoveTo(x,y);
  img.Canvas.LineTo(x1,y);
  img.Canvas.LineTo(x1+dx1,y1);
  img.Canvas.LineTo(x-dx2,y1);
  img.Canvas.LineTo(x,y);
end;

procedure TTrapezium.setDx1Dx2(adx1,adx2:integer);
begin
  dx1:=adx1;
  dx2:=adx2;
end;

//.  
procedure TTrapezium.MoveTo(ax, ay: integer);
begin
  Show;
  setXY(x+ax,y+ay);
  setX1Y1(x1+ax,y1+ay);
  limXY(x1-x,y1-y);
  Show;
end;

//.  
procedure TTrapezium.Resize(dx,dy,dx1,dx2:integer);
begin
  Show;
  setX1Y1(x1+dx,y1+dy);
  setDx1Dx2(dx1,dx2);
  limWH;
  Show;
end;

//.  
procedure TTrapezium.Delete;
begin
  Show;
end;
end.
